@@ -52,9 +52,12 @@ Datum pgq3c_ang2ipix_real(PG_FUNCTION_ARGS);
 Datum pgq3c_ipix2ang(PG_FUNCTION_ARGS);
 Datum pgq3c_pixarea(PG_FUNCTION_ARGS);
 Datum pgq3c_dist(PG_FUNCTION_ARGS);
+Datum pgq3c_dist_pm(PG_FUNCTION_ARGS);
 Datum pgq3c_sindist(PG_FUNCTION_ARGS);
+Datum pgq3c_sindist_pm(PG_FUNCTION_ARGS);
 Datum q3c_strquery(PG_FUNCTION_ARGS);
 Datum pgq3c_nearby_it(PG_FUNCTION_ARGS);
+Datum pgq3c_nearby_pm_it(PG_FUNCTION_ARGS);
 Datum pgq3c_ellipse_nearby_it(PG_FUNCTION_ARGS);
 Datum pgq3c_radial_array(PG_FUNCTION_ARGS);
 Datum pgq3c_radial_query_it(PG_FUNCTION_ARGS);
@@ -64,6 +67,7 @@ Datum pgq3c_in_ellipse(PG_FUNCTION_ARGS);
 Datum pgq3c_in_poly(PG_FUNCTION_ARGS);
 Datum pgq3c_get_version(PG_FUNCTION_ARGS);
 
+
 PG_FUNCTION_INFO_V1(pgq3c_get_version);
 Datum pgq3c_get_version(PG_FUNCTION_ARGS)
 {
@@ -230,6 +234,145 @@ Datum pgq3c_sindist(PG_FUNCTION_ARGS)
 }
 
 
+PG_FUNCTION_INFO_V1(pgq3c_sindist_pm);
+Datum pgq3c_sindist_pm(PG_FUNCTION_ARGS)
+{
+	q3c_coord_t pmra1 = 0, pmdec1 = 0, epoch1 =0 , epoch2 =0;
+	q3c_coord_t ra1, dec1, ra2, dec2, ra1_shift, dec1_shift;
+	bool pm_enabled = true;
+	q3c_coord_t res;
+	if (PG_ARGISNULL(0) || PG_ARGISNULL(1) || 
+		PG_ARGISNULL(5) || PG_ARGISNULL(6))
+	{
+		elog(ERROR, "The RA, DEC columns are not allowed to be null");
+	}
+
+	ra1 = PG_GETARG_FLOAT8(0);
+	dec1 = PG_GETARG_FLOAT8(1);
+
+	if (!PG_ARGISNULL(2)) 
+	{
+	    pmra1 = PG_GETARG_FLOAT8(2);
+	} 
+	else
+	{
+		pm_enabled = false;
+	}
+	
+	if (!PG_ARGISNULL(3)) 
+	{
+	    pmdec1 = PG_GETARG_FLOAT8(3);
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+	
+   	if (!PG_ARGISNULL(4))
+	{
+		epoch1 = PG_GETARG_FLOAT8(4);
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+	
+	ra2 = PG_GETARG_FLOAT8(5);
+	dec2 = PG_GETARG_FLOAT8(6);
+   	if (!PG_ARGISNULL(7))
+	{
+		epoch2 = PG_GETARG_FLOAT8(7);
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+       
+
+    if (pm_enabled)
+	{
+		ra1_shift = ra1 + pmra1 * (epoch2 - epoch1) / 3600000;
+		dec1_shift = dec1 + pmdec1 * (epoch2 - epoch1) / 3600000;
+	}
+	else
+	{
+		ra1_shift = ra1;
+		dec1_shift = dec1;
+	}
+	res = q3c_sindist(ra1_shift, dec1_shift, ra2, dec2);
+	PG_RETURN_FLOAT8(res);
+}
+
+
+PG_FUNCTION_INFO_V1(pgq3c_dist_pm);
+Datum pgq3c_dist_pm(PG_FUNCTION_ARGS)
+{
+	q3c_coord_t pmra1 = 0, pmdec1 = 0, epoch1 =0 , epoch2 =0;
+	q3c_coord_t ra1, dec1, ra2, dec2, ra1_shift, dec1_shift;
+	bool pm_enabled = true;
+	q3c_coord_t res;
+	if (PG_ARGISNULL(0) || PG_ARGISNULL(1) || 
+		PG_ARGISNULL(5) || PG_ARGISNULL(6))
+	{
+		elog(ERROR, "The RA, DEC columns are not allowed to be null");
+	}
+
+	ra1 = PG_GETARG_FLOAT8(0);
+	dec1 = PG_GETARG_FLOAT8(1);
+
+	if (!PG_ARGISNULL(2)) 
+	{
+	    pmra1 = PG_GETARG_FLOAT8(2);
+	} 
+	else
+	{
+		pm_enabled = false;
+	}
+	
+	if (!PG_ARGISNULL(3)) 
+	{
+	    pmdec1 = PG_GETARG_FLOAT8(3);
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+	
+   	if (!PG_ARGISNULL(4))
+	{
+		epoch1 = PG_GETARG_FLOAT8(4);
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+	
+	ra2 = PG_GETARG_FLOAT8(5);
+	dec2 = PG_GETARG_FLOAT8(6);
+   	if (!PG_ARGISNULL(7))
+	{
+		epoch2 = PG_GETARG_FLOAT8(7);
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+       
+
+    if (pm_enabled)
+	{
+		ra1_shift = ra1 + pmra1 * (epoch2 - epoch1) / 3600000;
+		dec1_shift = dec1 + pmdec1 * (epoch2 - epoch1) / 3600000;
+	}
+	else
+	{
+		ra1_shift = ra1;
+		dec1_shift = dec1;
+	}
+	res = q3c_dist(ra1_shift, dec1_shift, ra2, dec2);
+	PG_RETURN_FLOAT8(res);
+}
+
 
 PG_FUNCTION_INFO_V1(pgq3c_nearby_it);
 Datum pgq3c_nearby_it(PG_FUNCTION_ARGS)
@@ -289,6 +432,118 @@ Datum pgq3c_nearby_it(PG_FUNCTION_ARGS)
 }
 
 
+PG_FUNCTION_INFO_V1(pgq3c_nearby_pm_it);
+Datum pgq3c_nearby_pm_it(PG_FUNCTION_ARGS)
+{
+	q3c_ipix_t ipix_array[8];
+	static q3c_ipix_t ipix_array_buf[8];
+	static q3c_coord_t ra_cen_buf, dec_cen_buf, radius_buf;
+	static q3c_coord_t pmra_buf, pmdec_buf, max_epoch_delta_buf;
+	static int invocation;
+	int i;
+	extern struct q3c_prm hprm;
+	q3c_circle_region circle;
+	q3c_coord_t new_radius;
+	q3c_coord_t ra_cen, dec_cen, pmra=0, pmdec=0;
+	q3c_coord_t max_epoch_delta=0, radius=0 ;
+	bool pm_enabled = true;
+	int iteration; 
+	if (PG_ARGISNULL(0) || PG_ARGISNULL(1) || PG_ARGISNULL(6))
+	  {
+	    elog(ERROR, "Right Ascensions and raddii must be not null");
+	  }
+
+	ra_cen = PG_GETARG_FLOAT8(0); // ra_cen
+	dec_cen = PG_GETARG_FLOAT8(1); // dec_cen
+	if (!PG_ARGISNULL(2))
+	{	
+		pmra = PG_GETARG_FLOAT8(2); // pmra
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+	if (!PG_ARGISNULL(3))
+	{	
+		pmdec = PG_GETARG_FLOAT8(3); // pmdec
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+
+	if (!PG_ARGISNULL(4))
+	{	
+		max_epoch_delta = PG_GETARG_FLOAT8(4); 
+	}
+	else
+	{
+		pm_enabled = false;
+	}
+
+	radius = PG_GETARG_FLOAT8(5); // error radius
+	
+	iteration = PG_GETARG_INT32(6); // iteration
+	
+	if ( (!isfinite(ra_cen)) || (!isfinite(dec_cen)) )
+	{
+		elog(ERROR, "The values of ra,dec are infinites or NaNs");
+	}
+	if ((!pm_enabled) ||  (!isfinite(pmra)) || (!isfinite(pmdec)) || 
+		(!isfinite(max_epoch_delta)) )
+	  {
+	    pmra =  0;
+	    pmdec = 0;
+	    max_epoch_delta = 0;
+	  }
+	if (max_epoch_delta<0)
+	{
+		elog(ERROR, "The maximum epoch difference must be >=0 ");
+	}
+	if (invocation == 0)
+	/* If this is the first invocation of the function */
+	{
+	/* I should set invocation=1 ONLY!!! after setting ra_cen_buf, dec_cen_buf and
+   	 * ipix_buf. Because if the program will be canceled or crashed
+	 * for some reason the invocation should be == 0
+ 	 */
+	}
+	else
+	{
+	  if ((ra_cen == ra_cen_buf) && (dec_cen == dec_cen_buf) && 
+	  	(radius == radius_buf) && (pmra == pmra_buf) &&
+		(pmdec == pmdec_buf) && (max_epoch_delta == max_epoch_delta_buf))
+		{
+			PG_RETURN_INT64(ipix_array_buf[iteration]);
+		}
+	}
+	
+	new_radius = q3c_sqrt(pmra * pmra + pmdec * pmdec)/ 3600000 * max_epoch_delta + radius;
+
+	ra_cen = UNWRAP_RA(ra_cen);
+	if (q3c_fabs(dec_cen)>90) {dec_cen = q3c_fmod(dec_cen,90);}
+	circle.ra = ra_cen;
+	circle.dec = dec_cen;
+	circle.rad = new_radius;
+
+	q3c_get_nearby(&hprm, Q3C_CIRCLE, &circle, ipix_array);
+
+	for(i = 0; i < 8; i++)
+	{
+		ipix_array_buf[i] = ipix_array[i];
+	}
+
+	ra_cen_buf = ra_cen;
+	dec_cen_buf = dec_cen;
+	radius_buf = radius;
+	max_epoch_delta_buf = max_epoch_delta;
+	pmra_buf = pmra;
+	pmdec_buf = pmdec;
+	invocation=1;
+	PG_RETURN_INT64(ipix_array_buf[iteration]);
+}
+
+
 
 PG_FUNCTION_INFO_V1(pgq3c_ellipse_nearby_it);
 Datum pgq3c_ellipse_nearby_it(PG_FUNCTION_ARGS)
